#!/usr/bin/perl
#Released under the public domain.  Feel free to do with it as you wish!
use strict;
use Encode qw/decode/;
use Getopt::Std;
use File::Spec;

my %opts=();
getopts("tg:p:s:o:", \%opts);

my $pdf = $opts{"p"};
my $sql = $opts{"s"};
my $outputpdf = $opts{"o"};

my $pdffilename;

#if the sql file isn't specified, let's try to find it by assuming we've been pointed straight at
#the boox device
if (!$sql) {
	(my $volume,my $directories,$pdffilename) = File::Spec->splitpath( $pdf );
	print $volume." ".$directories." ".$pdffilename."\n";
	if($directories ne '') {
		$directories = File::Spec->catdir(($directories,'.onyx'));
	} else {
		$directories = '.onyx';
	}
	print $directories."\n";
	$sql = File::Spec->catpath($volume,$directories,"$pdffilename.sketch");
	print "No directory specified for sqlite file.  Trying the Onyx default: ".$sql."\n";
}

#generate an output file name if we aren't provided with one
if(!$outputpdf) {
	$outputpdf = substr($pdffilename,0,rindex($pdffilename,"."));
	$outputpdf = $outputpdf."-commented.pdf";
	print "No output file specified, using $outputpdf\n";
}
my $testrun = $opts{"t"}; #if set, only print out annotations
my $gslocation = $opts{"g"}; #if set, use custom location for ghostscript.  mostly intended for future windows support

if(!(-e $pdf) || !(-e $sql)) {
	die("USAGE: addannot -p <pdf file> [-s <sql file>] [-o <output pdf file>] [-g] [-t]\n-g specifies ghostscript location, while -t specifies that we should not attempt to generate the pdf - just print out a list of annotations"); 
}

my $page = 0;
my $lastline;
my $page;
my $annot;
my $samepagecount = 0;
my @elements;
my $tempfile = ".annotmp";

#find out what OS we're on, so we can use the right sqlite executable
my $sqliteloc;
if($^O =~ m/darwin/) {
	$sqliteloc = "mac/sqlite3"
} elsif ($^O =~ m/MSWin32/) {
        $sqliteloc = "win/sqlite3.exe"
} else {
        $sqliteloc = "lin/sqlite3"
}

#get all the annotation data from sqlite
my @data = `$sqliteloc -separator "---" \"$sql\" "select page_position, quote(data) from annotation ORDER BY page_position"`;


#create a nice array describing the page number and text of each annotation
foreach my $line (@data) {
	chomp $line;
	my @splitline = split(/---/,$line);
	my $pageno = $splitline[0]+1;
	my $allannots = $splitline[1];
	#get past some of the preamble in the blob, cut off the quote at the end
	my @annotset = split(/0023007000640066006C006F006300/, substr ($splitline[1], 24, length($splitline[1])-25));
	#every second pdfloc has our text after it, after which we want
	#to seek to a coule of characters past the next ')' character
	for (my $annotno = 2; $annotno < scalar(@annotset); $annotno+=2) {
		my $annot = $annotset[$annotno];
		$annot = substr($annot,index($annot,"2900")+12);
		my $findnull = index($annot,"0000");
		#last record has some garbage in it - strip it out
		if($findnull != -1) {
			$annot = substr($annot,0,index($annot,"0000"));
		}
		#convert the unpacked utf-16 info into a real string 
		my $packstr = 'H'.(length($annot)+4);
		$annot = decode("UTF-16LE", pack($packstr, $annot));
		print $pageno." ".$annot."\n";
		push (@elements,[$pageno,$annot]);
	}
}

#this prints out the config information that ghostscript needs to add our annotations to the PDF.  If you want the
#note graphics to appear in a different place, edit the Rect[ ... ] section.  Coordinates are bottom left -> top right
open TEMPFILE, ">", $tempfile or die $!;
foreach my $element (@elements) {
	print TEMPFILE '[ /Contents ('.$element->[1].') /Rect [70 250 170 350] /Subtype /Text /Name /Note /SrcPg '.$element->[0].' /Open true /ModDate (D:20120504183333) /Title ('.$element->[1].') /Color [.5 .5 0] /ANN pdfmark '."\n";
}

close TEMPFILE;

#command to call ghostscript with
if(!$testrun) {
	print "\n Launching ghostscript to add comments into your file!\n";
	if (defined $gslocation) {
		$gslocation = $gslocation;
	} else {
		$gslocation = "gs";
	}
	my $runstr = $gslocation." -o \"$outputpdf\" -sDEVICE=pdfwrite -f \"$pdf\" \"$tempfile\"";
	exec($runstr);
}
